@@ -11,7 +11,7 @@ ErrorHandler::ErrorHandler()
 
 void ErrorHandler::Clean()
 {
-  ExitCode=SUCCESS;
+  ExitCode=RARX_SUCCESS;
   ErrCount=0;
   EnableBreak=true;
   Silent=false;
@@ -22,20 +22,20 @@ void ErrorHandler::Clean()
 void ErrorHandler::MemoryError()
 {
   MemoryErrorMsg();
-  Throw(MEMORY_ERROR);
+  Throw(RARX_MEMORY);
 }
 
 
-void ErrorHandler::OpenError(const char *FileName)
+void ErrorHandler::OpenError(const char *FileName,const wchar *FileNameW)
 {
 #ifndef SILENT
   OpenErrorMsg(FileName);
-  Throw(OPEN_ERROR);
+  Throw(RARX_OPEN);
 #endif
 }
 
 
-void ErrorHandler::CloseError(const char *FileName)
+void ErrorHandler::CloseError(const char *FileName,const wchar *FileNameW)
 {
 #ifndef SILENT
   if (!UserBreak)
@@ -45,23 +45,23 @@ void ErrorHandler::CloseError(const char *FileName)
   }
 #endif
 #if !defined(SILENT) || defined(RARDLL)
-  Throw(FATAL_ERROR);
+  Throw(RARX_FATAL);
 #endif
 }
 
 
-void ErrorHandler::ReadError(const char *FileName)
+void ErrorHandler::ReadError(const char *FileName,const wchar *FileNameW)
 {
 #ifndef SILENT
-  ReadErrorMsg(NULL,FileName);
+  ReadErrorMsg(NULL,NULL,FileName,FileNameW);
 #endif
 #if !defined(SILENT) || defined(RARDLL)
-  Throw(FATAL_ERROR);
+  Throw(RARX_FATAL);
 #endif
 }
 
 
-bool ErrorHandler::AskRepeatRead(const char *FileName)
+bool ErrorHandler::AskRepeatRead(const char *FileName,const wchar *FileNameW)
 {
 #if !defined(SILENT) && !defined(SFX_MODULE) && !defined(_WIN_CE)
   if (!Silent)
@@ -76,32 +76,32 @@ bool ErrorHandler::AskRepeatRead(const char *FileName)
 }
 
 
-void ErrorHandler::WriteError(const char *ArcName,const char *FileName)
+void ErrorHandler::WriteError(const char *ArcName,const wchar *ArcNameW,const char *FileName,const wchar *FileNameW)
 {
 #ifndef SILENT
-  WriteErrorMsg(ArcName,FileName);
+  WriteErrorMsg(ArcName,ArcNameW,FileName,FileNameW);
 #endif
 #if !defined(SILENT) || defined(RARDLL)
-  Throw(WRITE_ERROR);
+  Throw(RARX_WRITE);
 #endif
 }
 
 
-#ifdef _WIN_32
-void ErrorHandler::WriteErrorFAT(const char *FileName)
+#ifdef _WIN_ALL
+void ErrorHandler::WriteErrorFAT(const char *FileName,const wchar *FileNameW)
 {
 #if !defined(SILENT) && !defined(SFX_MODULE)
   SysErrMsg();
   ErrMsg(NULL,St(MNTFSRequired),FileName);
 #endif
 #if !defined(SILENT) && !defined(SFX_MODULE) || defined(RARDLL)
-  Throw(WRITE_ERROR);
+  Throw(RARX_WRITE);
 #endif
 }
 #endif
 
 
-bool ErrorHandler::AskRepeatWrite(const char *FileName,bool DiskFull)
+bool ErrorHandler::AskRepeatWrite(const char *FileName,const wchar *FileNameW,bool DiskFull)
 {
 #if !defined(SILENT) && !defined(_WIN_CE)
   if (!Silent)
@@ -116,7 +116,7 @@ bool ErrorHandler::AskRepeatWrite(const char *FileName,bool DiskFull)
 }
 
 
-void ErrorHandler::SeekError(const char *FileName)
+void ErrorHandler::SeekError(const char *FileName,const wchar *FileNameW)
 {
 #ifndef SILENT
   if (!UserBreak)
@@ -126,7 +126,7 @@ void ErrorHandler::SeekError(const char *FileName)
   }
 #endif
 #if !defined(SILENT) || defined(RARDLL)
-  Throw(FATAL_ERROR);
+  Throw(RARX_FATAL);
 #endif
 }
 
@@ -148,55 +148,70 @@ void ErrorHandler::MemoryErrorMsg()
 }
 
 
-void ErrorHandler::OpenErrorMsg(const char *FileName)
+void ErrorHandler::OpenErrorMsg(const char *FileName,const wchar *FileNameW)
 {
-  OpenErrorMsg(NULL,FileName);
+  OpenErrorMsg(NULL,NULL,FileName,FileNameW);
 }
 
 
-void ErrorHandler::OpenErrorMsg(const char *ArcName,const char *FileName)
+void ErrorHandler::OpenErrorMsg(const char *ArcName,const wchar *ArcNameW,const char *FileName,const wchar *FileNameW)
 {
 #ifndef SILENT
-  Log(ArcName && *ArcName ? ArcName:NULL,St(MCannotOpen),FileName);
+  if (FileName!=NULL)
+    Log(ArcName,St(MCannotOpen),FileName);
   Alarm();
   SysErrMsg();
 #endif
 }
 
 
-void ErrorHandler::CreateErrorMsg(const char *FileName)
+void ErrorHandler::CreateErrorMsg(const char *FileName,const wchar *FileNameW)
 {
-  CreateErrorMsg(NULL,FileName);
+  CreateErrorMsg(NULL,NULL,FileName,FileNameW);
 }
 
 
-void ErrorHandler::CreateErrorMsg(const char *ArcName,const char *FileName)
+void ErrorHandler::CreateErrorMsg(const char *ArcName,const wchar *ArcNameW,const char *FileName,const wchar *FileNameW)
 {
 #ifndef SILENT
-  Log(ArcName && *ArcName ? ArcName:NULL,St(MCannotCreate),FileName);
+  if (FileName!=NULL)
+    Log(ArcName,St(MCannotCreate),FileName);
   Alarm();
-#if defined(_WIN_32) && !defined(_WIN_CE) && !defined(SFX_MODULE) && defined(MAX_PATH)
+
+#if defined(_WIN_ALL) && !defined(_WIN_CE) && defined(MAX_PATH)
+  CheckLongPathErrMsg(FileName,FileNameW);
+#endif
+
+  SysErrMsg();
+#endif
+}
+
+
+// Check the path length and display the error message if it is too long.
+void ErrorHandler::CheckLongPathErrMsg(const char *FileName,const wchar *FileNameW)
+{
+#if defined(_WIN_ALL) && !defined(_WIN_CE) && !defined (SILENT) && defined(MAX_PATH)
   if (GetLastError()==ERROR_PATH_NOT_FOUND)
   {
-    size_t NameLength=strlen(FileName);
-    if (!IsFullPath(FileName))
+    wchar WideFileName[NM];
+    GetWideName(FileName,FileNameW,WideFileName,ASIZE(WideFileName));
+    size_t NameLength=wcslen(WideFileName);
+    if (!IsFullPath(WideFileName))
     {
-      char CurDir[NM];
-      GetCurrentDirectory(sizeof(CurDir),CurDir);
-      NameLength+=strlen(CurDir)+1;
+      wchar CurDir[NM];
+      GetCurrentDirectoryW(ASIZE(CurDir),CurDir);
+      NameLength+=wcslen(CurDir)+1;
     }
     if (NameLength>MAX_PATH)
     {
-      Log(ArcName && *ArcName ? ArcName:NULL,St(MMaxPathLimit),MAX_PATH);
+      Log(NULL,St(MMaxPathLimit),MAX_PATH);
     }
   }
-#endif
-  SysErrMsg();
 #endif
 }
 
 
-void ErrorHandler::ReadErrorMsg(const char *ArcName,const char *FileName)
+void ErrorHandler::ReadErrorMsg(const char *ArcName,const wchar *ArcNameW,const char *FileName,const wchar *FileNameW)
 {
 #ifndef SILENT
   ErrMsg(ArcName,St(MErrRead),FileName);
@@ -205,7 +220,7 @@ void ErrorHandler::ReadErrorMsg(const char *ArcName,const char *FileName)
 }
 
 
-void ErrorHandler::WriteErrorMsg(const char *ArcName,const char *FileName)
+void ErrorHandler::WriteErrorMsg(const char *ArcName,const wchar *ArcNameW,const char *FileName,const wchar *FileNameW)
 {
 #ifndef SILENT
   ErrMsg(ArcName,St(MErrWrite),FileName);
@@ -214,7 +229,7 @@ void ErrorHandler::WriteErrorMsg(const char *ArcName,const char *FileName)
 }
 
 
-void ErrorHandler::Exit(int ExitCode)
+void ErrorHandler::Exit(RAR_EXIT ExitCode)
 {
 #ifndef SFX_MODULE
   Alarm();
@@ -231,7 +246,7 @@ void ErrorHandler::ErrMsg(const char *ArcName,const char *fmt,...)
   va_start(argptr,fmt);
   vsprintf(Msg,fmt,argptr);
   va_end(argptr);
-#ifdef _WIN_32
+#ifdef _WIN_ALL
   if (UserBreak)
     Sleep(5000);
 #endif
@@ -245,18 +260,18 @@ void ErrorHandler::ErrMsg(const char *ArcName,const char *fmt,...)
 #endif
 
 
-void ErrorHandler::SetErrorCode(int Code)
+void ErrorHandler::SetErrorCode(RAR_EXIT Code)
 {
   switch(Code)
   {
-    case WARNING:
-    case USER_BREAK:
-      if (ExitCode==SUCCESS)
+    case RARX_WARNING:
+    case RARX_USERBREAK:
+      if (ExitCode==RARX_SUCCESS)
         ExitCode=Code;
       break;
-    case FATAL_ERROR:
-      if (ExitCode==SUCCESS || ExitCode==WARNING)
-        ExitCode=FATAL_ERROR;
+    case RARX_FATAL:
+      if (ExitCode==RARX_SUCCESS || ExitCode==RARX_WARNING)
+        ExitCode=RARX_FATAL;
       break;
     default:
       ExitCode=Code;
@@ -267,7 +282,7 @@ void ErrorHandler::SetErrorCode(int Code)
 
 
 #if !defined(GUI) && !defined(_SFX_RTL_)
-#ifdef _WIN_32
+#ifdef _WIN_ALL
 BOOL __stdcall ProcessSignal(DWORD SigType)
 #else
 #if defined(__sun)
@@ -276,7 +291,9 @@ extern "C"
 void _stdfunction ProcessSignal(int SigType)
 #endif
 {
-#ifdef _WIN_32
+#ifdef _WIN_ALL
+  // When a console application is run as a service, this allows the service
+  // to continue running after the user logs off. 
   if (SigType==CTRL_LOGOFF_EVENT)
     return(TRUE);
 #endif
@@ -284,15 +301,15 @@ void _stdfunction ProcessSignal(int SigType)
   mprintf(St(MBreak));
   for (int I=0;!File::RemoveCreated() && I<3;I++)
   {
-#ifdef _WIN_32
+#ifdef _WIN_ALL
     Sleep(100);
 #endif
   }
 #if defined(USE_RC) && !defined(SFX_MODULE) && !defined(_WIN_CE) && !defined(RARDLL)
   ExtRes.UnloadDLL();
 #endif
-  exit(USER_BREAK);
-#if defined(_WIN_32) && !defined(_MSC_VER)
+  exit(RARX_USERBREAK);
+#if defined(_WIN_ALL) && !defined(_MSC_VER)
   // never reached, just to avoid a compiler warning
   return(TRUE);
 #endif
@@ -304,7 +321,7 @@ void ErrorHandler::SetSignalHandlers(bool Enable)
 {
   EnableBreak=Enable;
 #if !defined(GUI) && !defined(_SFX_RTL_)
-#ifdef _WIN_32
+#ifdef _WIN_ALL
   SetConsoleCtrlHandler(Enable ? ProcessSignal:NULL,TRUE);
 //  signal(SIGBREAK,Enable ? ProcessSignal:SIG_IGN);
 #else
@@ -315,9 +332,9 @@ void ErrorHandler::SetSignalHandlers(bool Enable)
 }
 
 
-void ErrorHandler::Throw(int Code)
+void ErrorHandler::Throw(RAR_EXIT Code)
 {
-  if (Code==USER_BREAK && !EnableBreak)
+  if (Code==RARX_USERBREAK && !EnableBreak)
     return;
   ErrHandler.SetErrorCode(Code);
 #ifdef ALLOW_EXCEPTIONS
@@ -332,31 +349,39 @@ void ErrorHandler::Throw(int Code)
 void ErrorHandler::SysErrMsg()
 {
 #if !defined(SFX_MODULE) && !defined(SILENT)
-#ifdef _WIN_32
-    #define STRCHR strchr
-    #define ERRCHAR char
-  ERRCHAR  *lpMsgBuf=NULL;
+#ifdef _WIN_ALL
+  wchar *lpMsgBuf=NULL;
   int ErrType=GetLastError();
   if (ErrType!=0 && FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER|FORMAT_MESSAGE_FROM_SYSTEM,
               NULL,ErrType,MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
               (LPTSTR)&lpMsgBuf,0,NULL))
   {
-    ERRCHAR  *CurMsg=lpMsgBuf;
+    wchar *CurMsg=lpMsgBuf;
     while (CurMsg!=NULL)
     {
       while (*CurMsg=='\r' || *CurMsg=='\n')
         CurMsg++;
       if (*CurMsg==0)
         break;
-      ERRCHAR *EndMsg=STRCHR(CurMsg,'\r');
+      wchar *EndMsg=wcschr(CurMsg,'\r');
       if (EndMsg==NULL)
-        EndMsg=STRCHR(CurMsg,'\n');
+        EndMsg=wcschr(CurMsg,'\n');
       if (EndMsg!=NULL)
       {
         *EndMsg=0;
         EndMsg++;
       }
-      Log(NULL,"\n%s",CurMsg);
+      // We use ASCII for output in Windows console, so let's convert Unicode
+      // message to single byte.
+      size_t Length=wcslen(CurMsg)*2; // Must be enough for DBCS characters.
+      char *MsgA=(char *)malloc(Length+2);
+      if (MsgA!=NULL)
+      {
+        WideToChar(CurMsg,MsgA,Length+1);
+        MsgA[Length]=0;
+        Log(NULL,"\n%s",MsgA);
+        free(MsgA);
+      }
       CurMsg=EndMsg;
     }
   }
@@ -364,9 +389,12 @@ void ErrorHandler::SysErrMsg()
 #endif
 
 #if defined(_UNIX) || defined(_EMX)
-  char *err=strerror(errno);
-  if (err!=NULL)
-    Log(NULL,"\n%s",err);
+  if (errno!=0)
+  {
+    char *err=strerror(errno);
+    if (err!=NULL)
+      Log(NULL,"\n%s",err);
+  }
 #endif
 
 #endif
