@@ -1,6 +1,5 @@
 #include "rar.hpp"
 
-static void WriteToLog(const char *ArcName,const char *Message);
 
 static char LogName[NM];
 
@@ -10,6 +9,7 @@ void InitLogOptions(char *LogName)
 }
 
 
+#ifndef SILENT
 void Log(const char *ArcName,const char *Format,...)
 {
   safebuf char Msg[2*NM+1024];
@@ -19,5 +19,6 @@ void Log(const char *ArcName,const char *Format,...)
   va_end(ArgPtr);
   eprintf("%s",Msg);
 }
+#endif
 
 
